Package: bayess
Title: Bayesian Essentials with R
Version: 1.6
Date: 2024-03-04
Depends: stats, mnormt, gplots, combinat
Author: Jean-Michel Marin [aut, cre],
        Christian P. Robert [aut]  
Maintainer: Jean-Michel Marin <jean-michel.marin@umontpellier.fr>
Description: Allows the reenactment of the R programs used in 
        the book Bayesian Essentials with R without further programming. 
        R code being available as well, they can be modified by the user
        to conduct one's own simulations. 
	    Marin J.-M. and Robert C. P. (2014) <doi:10.1007/978-1-4614-8687-9>.
URL: https://www.r-project.org, https://github.com/jmm34/bayess
License: GPL-2
NeedsCompilation: no
Packaged: 2024-03-04 10:30:19 UTC; marin
Repository: CRAN
Date/Publication: 2024-03-06 14:30:05 UTC
Built: R 4.3.3; ; 2025-01-24 14:44:36 UTC; unix
