import(mnormt)
import(gplots)
import(combinat)
importFrom(grDevices, heat.colors)
importFrom(graphics, contour, image, par)
importFrom(stats, approxfun, binomial, dgamma, dnorm, dpois, glm, integrate, pbeta, plogis, pnorm, poisson, qnorm, rbeta, rbinom, rgamma, rnbinom, rnorm, runif, var)
export(BayesReg)
export(ardipper)
export(gibbsnorm)
export(hmnoinfloglin)
export(loglinll)
export(rdirichlet)
export(ARllog)
export(gibbs)
export(hmnoinfprobit)
export(loglinnoinflpost)
export(pottsgibbs)
export(reconstruct)
export(ARmh)
export(MAmh)
export(hmflatlogit)
export(hmhmm)
export(MAllog)
export(solbeta)
export(BayesReg)
export(hmflatloglin)
export(isinghm)
export(ModChoBayesReg)
export(pottshm)
export(sumising)
export(gibbscap1)
export(hmflatprobit)
export(isingibbs)
export(pbino)
export(thresh)
export(gibbscap2)
export(pcapture)
export(probet)
export(truncnorm)
export(hmmeantemp)
export(logitll)
export(pdarroch)
export(probitll)
export(xneig4)
export(gibbsmean)
export(hmnoinflogit)
export(logitnoinflpost)
export(plotmix)
export(probitnoinflpost)
